/**************************************************************************
 * 2003-2016, Cypress Semiconductor Corporation or a subsidiary of Cypress 
 * Semiconductor Corporation.  All rights reserved.
 * 
 * This software, including source code, documentation and related materials 
 * (Software), is owned by Cypress Semiconductor Corporation or one of its 
 * subsidiaries (Cypress) and is protected by and subject to worldwide patent 
 * protection (United States and foreign), United States copyright laws and 
 * international treaty provisions. Therefore, you may use this Software only 
 * as provided in the license agreement accompanying the software package from 
 * which you obtained this Software (EULA).
 * 
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive, 
 * non-transferable license to copy, modify, and compile the Software source code 
 * solely for use in connection with Cypress?s integrated circuit products.  Any 
 * reproduction, modification, translation, compilation, or representation of 
 * this Software except as specified above is prohibited without the express 
 * written permission of Cypress.
 * 
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress 
 * reserves the right to make changes to the Software without notice. Cypress does 
 * not assume any liability arising out of the application or use of the Software 
 * or any product or circuit described in the Software. Cypress does not authorize 
 * its products for use in any products where a malfunction or failure of the Cypress 
 * product may reasonably be expected to result in significant property damage, injury 
 * or death (High Risk Product). By including Cypress?s product in a High Risk Product, 
 * the manufacturer of such system or application assumes all risk of such use and in 
 * doing so agrees to indemnify Cypress against all liability.
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <lld_test_util.h>
static  TestCaseTable_t StressTestCaseTable;
/*****************************\
 ****test case implementation******
\*****************************/
static bool cmd_stress_ChipNormalPgrmRead( void *d ,int argc, const char *argv[]){
    DeviceInfo* DevInfo = slld_GetGeoInfo();
    uint32_t loopno = strtoint(argv[1]);
    uint32_t ix,Sectorno;
    char arg1[30];
    const char *argvs[]={
            "cmd_func_NormalPgrmRead",
            arg1,/*SectorNo*/
            "all"
    };
    for(ix = 0; ix < loopno; ix++)
        for(Sectorno =0; Sectorno < DevInfo->SectorTotalNumber;Sectorno++){
            sprintf(arg1,"%d",Sectorno);
            if( !ok(cmd_func_NormalPgrmRead(NULL,3,argvs)))return false;
        }
    return true;
}

static bool cmd_stress_showstat( void *d, int argc, const char *argv[]){
    TestCaseTable_t *t =&StressTestCaseTable;
    int ix =0;
    tsc_printf("stress test case statistics :\n");
    for( ix= 0;ix < t->len;ix++){
        if(t->elems[ix].onOff== true){
            tsc_printf("[%-30s]:",t->elems[ix].name);
            tsc_printf("pass : %-5u ",t->elems[ix].pass);
            tsc_printf("error: %-5u \n",t->elems[ix].error);
            //		timerStruct start;            /*test start time ticks*/ TODO
        }

    }
    return true ;
}
static bool cmd_stress_cleanstat( void *d, int argc, const char *argv[]){
    TestCaseTable_t *t =&StressTestCaseTable;
    int ix =0;

    for( ix= 0;ix < t->len;ix++){
        t->elems[ix].pass = 0;
        t->elems[ix].error = 0;
        //t->elems[ix].start = 0; /*test start time ticks*/ TODO
    }
    tsc_printf("stress test case statistics cleaned:\n");
    return true ;
}

/*****************************\
 ****   Command support   ******
\*****************************/
static const Command_t StressTest_commands[] ={
        {"stress_ChipNormalPgrmRead",2,cmd_stress_ChipNormalPgrmRead,NULL,"loopNo","whole chip NormalPgrmRead"},
        {"stress_showstat",1,cmd_stress_showstat,NULL,NULL,"show stress test statistics"},
        {"stress_cleanstat",1,cmd_stress_cleanstat,NULL,NULL,"clean stress test statistics"},
};
CommandTable_t StressTest_commandsTable = {
        StressTest_commands,
        ElmNum(StressTest_commands)
};
/*****************************\
 ****   test case support   ******
\*****************************/

static TestCase_t stress_test_cases[] ={
        {true,"stress_ChipNormalPgrmRead",cmd_stress_ChipNormalPgrmRead,"whole chip NormalPgrmRead",2,"default 1 ",0,0},/*default parameters argc=1 < loopNo > */

};
static  TestCaseTable_t StressTestCaseTable ={
        stress_test_cases,
        ElmNum(stress_test_cases)
};

bool cmd_proc_stress_test(void * d ,int argc, const char *argv[]){
    Procedure_t *procedure = d;
    TestCaseTable_t *t = &StressTestCaseTable;
    int ix, argcs = 0;
    const char *argvs[MAX_INPUG_ARGS];
    char *buf = NULL;
    bool ret=true;
    //void * pDev = nand_get_mydev();
    //geoInfo->total_blocks = 8; // just for debug #define NUMB_OF_BLOCKS_IN_A_PLANE       4//1024
    for(ix = 0;ix < t->len; ix++) {

        if(true == t->elems[ix].onOff){
            buf = NULL;
            if(t->elems[ix].num_args && t->elems[ix].args){
                const int len= strlen(t->elems[ix].args);
                buf  = strcpy(malloc( len + 1),t->elems[ix].args);
                lld_printf("  [%u]. %s %s \n",ix+1,"description:",t->elems[ix].des);
                argcs = fmt_args(buf,argvs);
                if(argcs !=t->elems[ix].num_args){
                    lld_printf("  Fail ! \n");
                    ret =false;
                    lld_printf("default args not equal to the args number required\n");
                    free(buf);
                    continue;
                }
                PARAM_PRINT(argcs,argvs);
            }
            procedure->total++;
            if(t->elems[ix].call(&t->elems[ix],argcs,argvs)){
                procedure->pass++;
                t->elems[ix].pass++;
                lld_printf("  PASS ! \n");
            }
            else{
                procedure->error++;
                t->elems[ix].error++;
                lld_printf("  Fail ! \n");
                ret =false;
            }
            lld_printf("\n");
            if(buf)free(buf);
        }
    }
    cmd_stress_showstat(NULL,0,NULL);
    return ret;
}

