/* slld.h - Header file for Cypress SPI Low Level Driver */

/**************************************************************************
 * 2003-2016, Cypress Semiconductor Corporation or a subsidiary of Cypress 
 * Semiconductor Corporation.  All rights reserved.
 * 
 * This software, including source code, documentation and related materials 
 * (Software), is owned by Cypress Semiconductor Corporation or one of its 
 * subsidiaries (Cypress) and is protected by and subject to worldwide patent 
 * protection (United States and foreign), United States copyright laws and 
 * international treaty provisions. Therefore, you may use this Software only 
 * as provided in the license agreement accompanying the software package from 
 * which you obtained this Software (EULA).
 * 
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive, 
 * non-transferable license to copy, modify, and compile the Software source code 
 * solely for use in connection with Cypress's integrated circuit products.  Any 
 * reproduction, modification, translation, compilation, or representation of 
 * this Software except as specified above is prohibited without the express 
 * written permission of Cypress.
 * 
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress 
 * reserves the right to make changes to the Software without notice. Cypress does 
 * not assume any liability arising out of the application or use of the Software 
 * or any product or circuit described in the Software. Cypress does not authorize 
 * its products for use in any products where a malfunction or failure of the Cypress 
 * product may reasonably be expected to result in significant property damage, injury 
 * or death (High Risk Product). By including Cypress's product in a High Risk Product, 
 * the manufacturer of such system or application assumes all risk of such use and in 
 * doing so agrees to indemnify Cypress against all liability.
 */

#ifndef __INC_lldh
#define __INC_lldh

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define SLLD_VERSION          "16.2.1"

// ==================
// Status error flags
// ==================
#define HAS_NOTHING          0x00
#define HAS_STATBIT5_ERROR   0x01
#define HAS_STATBIT6_ERROR   0x02



#include "slld_targetspecific.h"

// =======================
// SPI Flash Commands info
// =======================
#define SPI_WRSR_CMD                (0x01)
#define SPI_PP_CMD                  (0x02)
#define SPI_READ_CMD                (0x03)
#define SPI_WRDI_CMD                (0x04)
#define SPI_RDSR_CMD                (0x05)
#define SPI_WREN_CMD                (0x06)
#define SPI_FAST_READ_CMD           (0x0B)
#define SPI_BULKERASE1_CMD                 (0x60)  // Bulk Erase
//#define SPI_CLSR2_CMD               (0x82)  //Clear Status Register 1 (alternate instruction) - Erase/Prog. Fail Reset
#define SPI_RDID_90_CMD  (0x90)
#define SPI_RDID_9F_CMD          (0x9F)
#define SPI_RDID_AB_CMD             (0xAB)
#define SPI_RES_CMD                 (0xAB)
#define SPI_SP_CMD                  (0xB9)
#define SPI_DP_CMD                  (0xB9)
#define SPI_BRAC_CMD                (0xB9)
#define SPI_BULKERASE_CMD                  (0xC7)  // BULK ERASE
#define SPI_SE_CMD                  (0xD8)

/* Voltage States */
#define SLLD_P_VIL                  (0x0001)
#define SLLD_P_VIH                  (0x0002)

/* LLD System Specific Typedefs */
typedef unsigned short LLD_UINT16;  /* 16 bits wide */
typedef unsigned long  LLD_UINT32;  /* 32 bits wide */
typedef LLD_UINT32     ADDRESS;     /* Used for system level addressing */

/* SLLD System Specific Typedefs */
typedef unsigned char  BYTE;   /* 8 bits wide */
typedef unsigned short WORD;   /* 16 bits wide */
typedef unsigned long  DWORD;  /* 32 bits wide */
typedef BYTE FLASHDATA;

/* SLLD Internal Data Types */
//typedef DWORD PARAM;            /* MUST be at least 32 bits wide */
typedef unsigned long BYTECOUNT;  /* used for multi-byte operations */

/* boolean macros */
#ifndef TRUE
#define TRUE  (1)
#endif
#ifndef FALSE
#define FALSE (0)
#endif

typedef enum
{
    VOLATILE_WR = 0,
    NON_VOLATILE_WR
}WRR_mode;

/* data mask */
#define B0_MASK                  (0x01)
#define B1_MASK                  (0x02)
#define B2_MASK                  (0x04)
#define B3_MASK                  (0x08)
#define B5_MASK                  (0x20)
#define B6_MASK                  (0x40)
#define B7_MASK                  (0x80)

#define LLD_DEV_READ_MASK        (0xFF)
#define LLD_BYTES_PER_OP         (0x01)

/*RDAR read any register address definition*/
// Non-Volatile Status and Configuration Registers
#define    SR1NV         0x00000000
#define    CR1NV         0x00000002
#define    CR2NV         0x00000003
#define    CR3NV         0x00000004
#define    CR4NV         0x00000005
#define    NVDLP         0x00000005
// Non-Volatile Data Learning Register
#define    NVDLR         0x00000010
#define    PASS7_0       0x00000020
//Non-Volatile Password Register
#define    PASS15_8      0x00000021
#define    PASS23_160    0x00000022
#define    PASS31_24     0x00000023
#define    PASS39_32     0x00000024
#define    PASS47_40     0x00000025
#define    PASS55_48     0x00000026
#define    PASS63_56     0x00000027
//#ifdef FL_L
#define    IRP7_0           0x00000030
#define    IRP15_8         0x00000031
//#else
#define    ASPR7_0       0x00000030
#define    ASPR15_8      0x00000031
//#endif
#define    PRPRA15_A8       0x00000039
#define    PRPRA23_A16     0x0000003A
#define    PRPRA31_A24     0x0000003B
#define    SR1V          0x00800000
//Volatile Status and Configuration Registers
#define    SR2V          0x00800001
#define    CR1V          0x00800002
#define    CR2V          0x00800003
#define    CR3V          0x00800004
#define    CR4V          0x00800005
#define    VDLP          0x00800005
// Volatile Data Learning Register
#define    VDLR          0x00800010
//Volatile PPB Lock Register
#define    PPBL          0x00800040
#define    PR              0x00800040

/*Device status */
/*Flash Software protect status */
typedef enum {
    FLASH_SOFTWARE_UNPROTECTED = 0,
    FLASH_SOFTWARE_PROTECTED
} DEV_SOFTWARE_PROTECT_STATUS;

// Flash embedded operation status
typedef enum {
    dev_status_unknown = 0,
    dev_not_busy,
    dev_program_error,
    dev_erase_error,
    dev_suspend,
    dev_busy
} DEVSTATUS;

// SLLD Returned values
typedef enum {
    SLLD_OK = 0x0,
    SLLD_E_DEVICE_SOFTWARE_PROTECTED,
    SLLD_E_HAL_ERROR = 0x200,
    SLLD_ERROR = 0xFFFF
} SLLD_STATUS;

// FLASH_RD and FLASH_WR functions needed defines
#define ADDRESS_NOT_USED 0xFFFFFFFF
#define BUFFER_NOT_USED  (BYTE*)0

extern BYTE modebit_char;

// Include Nor Super Tests
//#define NST_TESTS

/* Include for Competitive Analysis Test */
/* #define CA_TEST */

/* public function prototypes */

/* Operation Functions */











extern SLLD_STATUS slld_Read_IDCmd
(
        BYTE  device_num,            /* device number */
        BYTE    *target                       /* variable in which to store read data */
);

extern SLLD_STATUS slld_RDIDCmd
(
        BYTE  device_num,            /* device number */
        BYTE      *target,                    /* variable in which to store read data */
        BYTECOUNT  len_in_bytes               /* number of bytes to read */
);


/* Registers Operations */

extern SLLD_STATUS slld_RDSRCmd
(
        BYTE  device_num,            /* device number */
        BYTE    *target                       /* variable in which to store read data */
);






extern SLLD_STATUS slld_WRENCmd (BYTE device_num); /* device number */

extern SLLD_STATUS slld_WRDICmd (BYTE device_num); /* device number */

extern SLLD_STATUS slld_WRSRCmd
(
        BYTE  device_num,            /* device number */
        BYTE   *data_buf                      /* variable containing data to program */
);

extern SLLD_STATUS slld_WRSROp
(
        BYTE  device_num,            /* device number */
        BYTE       *data_buf,                 /* variable containing data to program */
        DEVSTATUS  *dev_status_ptr            /* variable to store device status */
);

extern void slld_VersionCmd (void); /* device number */









/* Power Saving Mode Operation */
extern SLLD_STATUS slld_DPCmd (BYTE device_num); /* device number */

extern SLLD_STATUS slld_BRACCmd (BYTE device_num); /* device number */

extern SLLD_STATUS slld_SPCmd (BYTE device_num); /* device number */

extern SLLD_STATUS slld_RESCmd (BYTE device_num); /* device number */










/* Command Functions */

/* Read Operations */

extern SLLD_STATUS slld_ReadCmd
(
        BYTE  device_num,            /* device number */
        ADDRESS     sys_addr,                 /* device address given by system */
        BYTE       *target,                   /* variable in which to store read data */
        BYTECOUNT   len_in_bytes              /* number of bytes on which to operate */
);


extern SLLD_STATUS slld_Fast_ReadCmd
(
        BYTE  device_num,            /* device number */
        ADDRESS     sys_addr,                 /* device address given by system */
        BYTE       *target,                   /* variable in which to store read data */
        BYTECOUNT   len_in_bytes              /* number of bytes on which to operate */
);


























/* Erase Operations */

extern SLLD_STATUS slld_BECmd (BYTE device_num); /* device number */

extern SLLD_STATUS slld_BEOp
(
        BYTE  device_num,            /* device number */
        DEVSTATUS  *dev_status_ptr            /* variable to store device status */
);

extern SLLD_STATUS slld_BE1Cmd (BYTE device_num); /* device number */

extern SLLD_STATUS slld_BE1Op
(
        BYTE  device_num,            /* device number */
        DEVSTATUS  *dev_status_ptr            /* variable to store device status */
);

extern SLLD_STATUS slld_SECmd
(
        BYTE  device_num,            /* device number */
        ADDRESS  sys_addr                     /* device address given by system */
);

extern SLLD_STATUS slld_SEOp
(
        BYTE  device_num,            /* device number */
        ADDRESS     sys_addr,                 /* device address given by system */
        DEVSTATUS  *dev_status_ptr            /* variable to store device status */
);








/* Program Operations */

extern SLLD_STATUS slld_PPCmd
(
        BYTE  device_num,            /* device number */
        ADDRESS      sys_addr,                /* device address given by system */
        BYTE        *data_buf,                /* variable containing data to program */
        BYTECOUNT    len_in_bytes             /* number of bytes on which to operate */
);

extern SLLD_STATUS slld_PPOp
(
        BYTE  device_num,            /* device number */
        ADDRESS     sys_addr,                 /* device address given by system */
        BYTE       *data_buf,                 /* variable containing data to program */
        BYTECOUNT   len_in_bytes,             /* number of bytes on which to operate */
        DEVSTATUS  *dev_status_ptr            /* variable to store device status */
);











extern SLLD_STATUS slld_Poll
(
        BYTE  device_num,            /* device number */
        DEVSTATUS  *dev_status_ptr            /* variable to store device status */
);

extern SLLD_STATUS slld_StatusGet
(
        BYTE  device_num,            /* device number */
        DEVSTATUS  *dev_status_ptr            /* variable to store device status */
);

#ifdef INCLUDE_SOFTWARE_PROTECT_STATUS_CHECK
extern SLLD_STATUS slld_SoftwareProtectStatusGet
(
        DEV_SOFTWARE_PROTECT_STATUS*  dev_softwareprotect_status_ptr
        /* variable to store device software protect status */
);
#endif // INCLUDE_SOFTWARE_PROTECT_STATUS_CHECK

extern SLLD_STATUS slld_BufferedProgramOp
(
        BYTE  device_num,            /* device number */
        ADDRESS    sys_addr,                  /* device address given by system */
        BYTE      *data_buf,                  /* variable containing data to program */
        BYTECOUNT  len_in_bytes,              /* number of bytes on which to operate */
        DEVSTATUS *dev_status_ptr             /* variable to store device status */
);


extern SLLD_STATUS slld_BlockProtectOp
(
        BYTE  device_num,            /* device number */
        BYTE        bpb_value,                /* block protect bit value */
        DEVSTATUS  *dev_status_ptr            /* variable to store device status */
);


































































/*****************************************************
 * Define Flash read/write macro to be used by SLLD   *
 *****************************************************/
#ifdef TRACE
#define FLASH_WR(dn, c,a,d,n)    FlashWrite(dn, c,a,d,n)
#define FLASH_RD(dn, c,a,d,n)    FlashRead(dn, c,a,d,n)
#elif defined BOARD226
#define FLASH_WR(dn, c,a,d,n)    FLASH_WRITE(dn, c,a,d,n)
#define FLASH_RD(dn, c,a,d,n)    FLASH_READ(dn, c,a,d,n)
#else
#define FLASH_WR(dn,c,a,d,n)    FLASH_WRITE(dn,c,a,d,n)
#define FLASH_RD(dn,c,a,d,n)    FLASH_READ(dn,c,a,d,n)
#endif

#ifdef UCHAR
typedef unsigned char   UCHAR;
#endif
#ifdef USHORT
typedef unsigned short  USHORT;
#endif
#ifdef UINT
typedef unsigned int    UINT;
#endif
#ifdef ULONG
typedef unsigned long   ULONG;
#endif
#ifdef BOARD226
#include "slld_hal_226.h"
#endif

extern SLLD_STATUS slld_ReadOp
(
        BYTE  device_num,                     /* device number */
        ADDRESS     sys_addr,                 /* device address given by system */
        BYTE       *target,                   /* variable in which to store read data */
        BYTECOUNT   len_in_bytes              /* number of bytes to read */

);

extern SLLD_STATUS slld_WriteOp
(
        BYTE  device_num,                     /* device number */
        ADDRESS     sys_addr,                 /* device address given by system */
        BYTE       *data_buf,                 /* variable containing data to program */
        BYTECOUNT   len_in_bytes,             /* number of bytes to program */
        DEVSTATUS  *dev_status_ptr            /* variable to store device status */
);
/*****************************************************************************/

extern SLLD_STATUS slld_GetDevNumFromAddr
(
        ADDRESS     sys_addr,                  /* device address given by system */
        BYTE  *device_num                        /* device number */
);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __INC_lldh */
