/* trace.c - Source Code for Cypress's LLD Virtual Logic Analyzer trace support */
/**************************************************************************
 * 2003-2016, Cypress Semiconductor Corporation or a subsidiary of Cypress 
 * Semiconductor Corporation.  All rights reserved.
 * 
 * This software, including source code, documentation and related materials 
 * (Software), is owned by Cypress Semiconductor Corporation or one of its 
 * subsidiaries (Cypress) and is protected by and subject to worldwide patent 
 * protection (United States and foreign), United States copyright laws and 
 * international treaty provisions. Therefore, you may use this Software only 
 * as provided in the license agreement accompanying the software package from 
 * which you obtained this Software (EULA).
 * 
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive, 
 * non-transferable license to copy, modify, and compile the Software source code 
 * solely for use in connection with Cypress's integrated circuit products.  Any 
 * reproduction, modification, translation, compilation, or representation of 
 * this Software except as specified above is prohibited without the express 
 * written permission of Cypress.
 * 
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress 
 * reserves the right to make changes to the Software without notice. Cypress does 
 * not assume any liability arising out of the application or use of the Software 
 * or any product or circuit described in the Software. Cypress does not authorize 
 * its products for use in any products where a malfunction or failure of the Cypress 
 * product may reasonably be expected to result in significant property damage, injury 
 * or death (High Risk Product). By including Cypress's product in a High Risk Product, 
 * the manufacturer of such system or application assumes all risk of such use and in 
 * doing so agrees to indemnify Cypress against all liability.
 */

#ifdef __SLLD
#include "slld.h"
#elif __OLLD
#include "olld.h"
#elif __D2LLD
#include "d2lld.h"
#include "d2lld_hal.h"
#elif __OPLLD
#include "basic_lld.h"
#else
#include "lld.h"
#endif
#include "trace.h"

unsigned int trace_sync = 0;
extern unsigned long os_GetCurrentTicks();

#define STORE_TIMESTAMP     trace[cycle].timestamp = os_GetCurrentTicks()


#define LLD_DEV_FLASH
//#define LLD_DEV_SIM


CYCLE trace[MAX_CYCLES];
int cycle = -1;
int trace_wrapped = FALSE;

#ifdef __SLLD
void StoreTraceCycle(BYTE device_num,OPERATION op, FLASHDATA command, ADDRESS sys_addr, FLASHDATA data, int Number_Of_Bytes)
{
    cycle++;
    if (cycle >= MAX_CYCLES)
    {
        cycle = 0;
        trace_wrapped = TRUE;
    }

    trace[cycle].trace_sync      = trace_sync++;
    trace[cycle].device_num    = device_num;
    trace[cycle].op              = op;
    trace[cycle].command         = command;
    trace[cycle].sys_addr        = sys_addr;
    trace[cycle].data            = data;
    trace[cycle].Number_Of_Bytes = Number_Of_Bytes;
}

SLLD_STATUS FlashWrite(BYTE device_num, BYTE command, ADDRESS sys_addr, FLASHDATA *data_buffer, int Number_Of_Written_Bytes)
{
    SLLD_STATUS status;

    status = FLASH_WRITE(device_num, command, sys_addr, data_buffer, Number_Of_Written_Bytes);
    if (data_buffer == BUFFER_NOT_USED)
        StoreTraceCycle(device_num, TR_OP_WRITE, command, sys_addr, (FLASHDATA)0xFFFFFFFF, Number_Of_Written_Bytes);
    else
        StoreTraceCycle(device_num, TR_OP_WRITE, command, sys_addr, *data_buffer, Number_Of_Written_Bytes);

#ifdef PRINT_TIMESTAMP
    STORE_TIMESTAMP;
#endif

    return(status);
}

SLLD_STATUS FlashRead(BYTE device_num, BYTE command, ADDRESS sys_addr, FLASHDATA *data_buffer, int Number_Of_Read_Bytes)
{
    SLLD_STATUS status;

    status = FLASH_READ(device_num, command, sys_addr, data_buffer, Number_Of_Read_Bytes);
    if (data_buffer == BUFFER_NOT_USED)
        StoreTraceCycle(device_num, TR_OP_READ, command, sys_addr, (FLASHDATA)0xFFFFFFFF, Number_Of_Read_Bytes);
    else
        StoreTraceCycle(device_num, TR_OP_READ, command, sys_addr, *data_buffer, Number_Of_Read_Bytes);

#ifdef PRINT_TIMESTAMP
    STORE_TIMESTAMP;
#endif

    return(status);
}
#elif !(defined __D2LLD)
void StoreTraceCycle(BYTE device_num, OPERATION op, FLASHDATA * base, ADDRESS offset, ADDRESS addr, FLASHDATA data)
{
    cycle++;
    if (cycle >= MAX_CYCLES)
    {
        cycle = 0;
        trace_wrapped = TRUE;
    }
    trace[cycle].trace_sync = trace_sync++;
    trace[cycle].device_num    = device_num;
    trace[cycle].op         = op;
    trace[cycle].base       = base;
    trace[cycle].offset     = offset;
    trace[cycle].addr       = addr;
    trace[cycle].data       = data;

}

#ifdef LLD_DEV_FLASH
void FlashWrite(BYTE device_num, FLASHDATA * addr, ADDRESS offset, FLASHDATA data)
{
#ifdef EXTEND_ADDR
    FlashWrite_Extend(device_num, addr, offset, data);
    StoreTraceCycle(device_num, TR_OP_WRITE, addr, offset, (ADDRESS)((volatile FLASHDATA*)(addr) + (offset)), data);
#else
    FLASH_OFFSET((addr),(offset)) = (data);
    StoreTraceCycle(device_num, TR_OP_WRITE, addr, offset, (ADDRESS)((volatile FLASHDATA*)(addr) + (offset)), data);
#endif

#ifdef PRINT_TIMESTAMP
    STORE_TIMESTAMP;
#endif
}
#endif  //LLD_DEV_FLASH

#ifdef LLD_DEV_FLASH
FLASHDATA FlashRead(BYTE device_num, FLASHDATA * addr, ADDRESS offset)
{
    FLASHDATA data = 0;

#ifdef EXTEND_ADDR
    data = FlashRead_Extend(device_num, addr, offset);
    StoreTraceCycle(device_num, TR_OP_READ, addr, offset, (ADDRESS)((volatile FLASHDATA*)(addr) + (offset)), data);
#else
    data = FLASH_OFFSET((addr),(offset));
    StoreTraceCycle(device_num, TR_OP_READ, addr, offset, (ADDRESS)((volatile FLASHDATA*)(addr) + (offset)), data);
#endif

#ifdef PRINT_TIMESTAMP
    STORE_TIMESTAMP;
#endif

    return(data);
}
#endif // LLD_DEV_FLASH

#else //_D2LLD

void StoreTraceCycle(BYTE device_num, OPERATION op, FLASHDATA * base, ADDRESS offset, FLASHDATA * src, BYTECOUNT size)
{
    int i = size;

    while(i > 0)
    {
        cycle++;
        if (cycle >= MAX_CYCLES)
        {
            cycle = 0;
            trace_wrapped = TRUE;
        }
        trace[cycle].trace_sync = trace_sync++;
        trace[cycle].device_num    = device_num;        
        trace[cycle].op         = op;
        trace[cycle].base       = base;
        trace[cycle].offset     = offset;
        trace[cycle].src       = *src;
        trace[cycle].size       = LLD_BYTES_PER_OP;

        i -= LLD_BYTES_PER_OP;
        offset += LLD_BYTES_PER_OP;
        src++ ;
    }
}

void FlashWrite(BYTE device_num, FLASHDATA * addr, ADDRESS offset, FLASHDATA * src, BYTECOUNT size)
{
    d2lld_HalWrite(device_num, addr, offset, src, size);
    StoreTraceCycle(device_num, TR_OP_WRITE, addr, offset, src, size);

#ifdef PRINT_TIMESTAMP
    STORE_TIMESTAMP;
#endif

}

void FlashRead(BYTE device_num, FLASHDATA * addr, ADDRESS offset, FLASHDATA * buf, BYTECOUNT size)
{
    d2lld_HalRead(device_num, addr, offset, buf, size);
    StoreTraceCycle(device_num, TR_OP_READ, addr, offset, buf, size);

#ifdef PRINT_TIMESTAMP
    STORE_TIMESTAMP;
#endif
}

#endif // _D2LLD

#ifdef TRACE_NAND_SIGNAL
void FlashRecordSetSignal(BYTE device_num, LLD_UINT32 sig_no, LLD_UINT32 data)
{
    StoreTraceCycle(device_num, OP_SET_SIGNAL, 0, 0, (ADDRESS)sig_no, (FLASHDATA)data);

#ifdef PRINT_TIMESTAMP
    STORE_TIMESTAMP;
#endif
}

void FlashRecordGetSignal(BYTE device_num, LLD_UINT32 sig_no, LLD_UINT32 data)
{
    StoreTraceCycle(device_num, OP_GET_SIGNAL, 0, 0, (ADDRESS)sig_no, (FLASHDATA)data);

#ifdef PRINT_TIMESTAMP
    STORE_TIMESTAMP;
#endif
}
#endif  /* TRACE_NAND_SIGNAL */

